#!/bin/env pyton3

import json
import logging
import requests
import importlib
import xmltodict
from utils import parse_dict_with_extracts, extract_from_jsonpath
from colorama import init, Fore, Back, Style

class APICase(json.JSONEncoder):

    def __init__(self, config, name, url, method):
        """
        init
        """
        self.config = config
        self.name = name
        self.url = url
        self.method = method.upper()
        self.headers = config.HEADERS.copy()

    def set_name(self, name):
        """
        set name
        """
        self.name = name

    def set_url(self, url):
        """
        set url
        """
        self.url = url

    def set_method(self, method):
        """
        set method
        """
        self.method = method

    def set_headers(self, headers):
        """
        set headers
        """
        if headers:
            for _, key in enumerate(headers):
                self.headers[key] = str(headers[key])

    def set_header(self, key, value):
        """
        set header
        """
        self.headers[key] = str(value)

    def set_data(self, data):
        """
        set raw data
        """
        self.data = data if data else None

    def set_assertions(self, assertions):
        """
        set assertons
        """
        self.assertions = assertions if assertions else {}

    def set_extracts(self, extracts):
        """
        set extracts
        """
        self.extracts = extracts if extracts else {}

    def set_response(self, response):
        """
        set response
        """
        self.response = response

    def get_name(self):
        """
        get name/title
        """
        return self.name

    def get_url(self, is_parsed=False):
        """
        get url
        """
        if is_parsed:
            return parse_dict_with_extracts(self.config, self.url, '')
        return self.url

    def get_headers(self, is_parsed=False):
        """
        get headers
        """
        if is_parsed:
            return parse_dict_with_extracts(self.config, self.headers, '')
        return self.headers

    def get_data(self, is_parsed=False):
        """
        get raw data
        """
        if is_parsed:
            return parse_dict_with_extracts(self.config, self.data, '')
        return self.data

    def get_assertions(self, is_parsed=False):
        """
        get assertions
        """
        if is_parsed:
            return parse_dict_with_extracts(self.config, self.assertions, '')
        return self.assertions

    def get_extracts(self, is_parsed=False):
        """
        get extracts
        """
        if is_parsed:
            return parse_dict_with_extracts(self.config, self.extracts, '')
        return self.extracts

    def get_response(self):
        """
        get response
        """
        return self.response

    def do_request(self):
        """
        send request
        """
        headers = self.get_headers(True)
        url = self.get_url(True)
        data = self.get_data(True)

        response = None
        try:
            if (isinstance(data, dict)):
                response = requests.request(self.method, headers=headers, url=url, json=data)
            else:
                response = requests.request(self.method, headers=headers, url=url, data=data)
        except Exception as e:
            logging.error(Fore.RED + "    ERROR HTTP message:{}".format(e, ) + Fore.RESET)

        self.set_response(response)

    def parse_status_code(self):
        """
        parse response code
        """
        response = self.get_response()
        if response is None:
            return None

        return response.status_code
        
    def parse_response_headers(self):
        """
        parse response headers
        """
        response = self.get_response()
        if response is None:
            return None

        return dict(response.headers)

    def parse_response_body(self):
        """
        parse response
        """
        response = self.get_response()
        if response is None:
            return None
        
        resp_type = response.headers.get('Content-Type')
        if not resp_type:
            resp_type = str(self.get_headers().get('Content-Type'))
        
        try:
            if "xml" in resp_type:
                json_obj = xmltodict.parse(response.text)
            else:
                json_obj = json.loads(response.text)
            return json_obj
        except:
            # logging.error(Fore.RED + "    ERROR parsing response:" + resp.text + Fore.RESET)
            pass

        if response:
            return response.text

        return None

    def extractor(self, json_obj):
        """
        extract variables from response
        """
        exts = self.get_extracts()
        for _, var in enumerate(exts):
            v = exts[var]
            # extract from code/headers/body
            key = 'body'
            jp = v
            if isinstance(v, dict):
                key = v['in']
                jp = v['exp']
            self.config.EXTRACTS[var] = extract_from_jsonpath(jp, json_obj, key)
            logging.info("\t\tEXTRACT [{}]={}".format(var, self.config.EXTRACTS[var], ))

    def assertor(self, json_obj):
        """
        assertor
        """
        module_obj = None
        for _, assertion in enumerate(self.get_assertions()):
            comp = assertion['comparator']

            # load assertor
            module_name = "assertors." + comp
            func_name = "assert_" + comp
            module_obj = importlib.import_module(module_name)
            func = getattr(module_obj, func_name)
            # run real assertor
            func(self.config, self, assertion, json_obj)
        logging.info("")

    def __repr__(self) -> str:
        return json.dumps({
            "name": self.name,
            "url": self.url,
            "method": self.method,
            "headers": self.headers,
            "data": self.data,
            "extracts": self.extracts,
            "assertions": self.assertions,
        })


#!/bin/env python3

import logging
from jsonpath_ng import parse as parse_jsonpath
from colorama import init, Fore, Back, Style

def parse_dict_with_extracts(config, obj, key=None):
    """
    parse api with extracts
    """
    if isinstance(obj, (int, float)):
        obj = obj
        return obj
    
    if isinstance(obj, str):
        old = obj
        for K in config.EXTRACTS:
            V = config.EXTRACTS[K]
            place_holder = '{{' + K + '}}'
            if place_holder in obj:
                if V is not None:
                    obj = obj.replace(place_holder, V)
                    logging.debug(Fore.GREEN + "\t\tParsed {}:{} to {}".format(key, old, obj, ) + Fore.RESET)
                else:
                    logging.warn(Fore.RED + "\t\tParsed FAIL {}:{}".format(key, old, ) + Fore.RESET)
        return obj

    if not isinstance(obj, dict):
        return obj
    
    for k in obj:
        obj[k] = parse_dict_with_extracts(config, obj[k], k)
    return obj

def extract_from_jsonpath(jp, json_obj, key):
    """
    extract variables from response by jsonpath
    """
    if jp:
        if '$' in str(jp):
            try:
                if key:
                    matches = parse_jsonpath(jp).find(json_obj[key])
                else:
                    matches = parse_jsonpath(jp).find(json_obj)
                
                if len(matches) > 0:
                    return str(matches[0].value)
                return None
            except:
                # logging.error('ERROR ' + jp + ' jsonpath not found in ' + str(json_obj))
                return None
        else:
            return str(jp)
    return str(jp)


#!/bin/env python3

import logging
from utils import extract_from_jsonpath, parse_dict_with_extracts
from colorama import init, Fore, Back, Style

def assert_contains(config, apicase, assertion, json_obj):
    key = assertion.get('in', 'body')
    exp1 = extract_from_jsonpath(parse_dict_with_extracts(config, assertion['exp1']), json_obj, key)
    exp2 = extract_from_jsonpath(parse_dict_with_extracts(config, assertion['exp2']), json_obj, key)

    if exp1 and (exp2 in exp1):
        message = "{} = {} [contains] {}".format(assertion['exp1'], exp1, exp2, )
        assertion['message'] = message
        assertion['result'] = True
        logging.info("\t\t" + Fore.GREEN + message + Fore.RESET)
        return True
    
    message = "{} = {} [contains] {}".format(assertion['exp1'], exp1, exp2, )
    assertion['message'] = message
    assertion['result'] = False
    logging.warning("\t\t" + Fore.RED + message + Fore.RESET)
    return False


#!/bin/env python3
# encoding=utf8

import os
import sys
import json
import glob
import yaml
from models.apicase import APICase
from colorama import init, Fore, Back, Style
import logging
import argparse
import importlib
from utils import parse_dict_with_extracts, extract_from_jsonpath

config = None

def initial():
    # Initializes Colorama
    init(autoreset=True)

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-c', '--configfile',
        help="Config py file",
        type=str, dest="configfile",
        default="config.py"
    )
    parser.add_argument(
        '-t', '--testcases',
        help="Testcases yaml files directory",
        type=str, dest="testcases",
        default="./testcases"
    )
    parser.add_argument(
        '-o', '--output',
        help="Output file of test result location",
        type=str, dest="output",
        default="result.json"
    )
    parser.add_argument(
        '-v', '--verbose',
        help="Show verbose info",
        action="count",
        default=1
    )
    args = parser.parse_args()
    loglevel = logging.WARNING - args.verbose * 10
    logging.basicConfig(level=loglevel, format='%(message)s')
    logging.getLogger("requests").setLevel(logging.CRITICAL)
    logging.getLogger("urllib3").setLevel(logging.CRITICAL)

    config_path = os.path.dirname(os.path.abspath(args.configfile))
    if not os.path.isfile(args.configfile):
        logging.error(Fore.RED + "Config file not exists:" + args.configfile + Fore.RESET)
        sys.exit(1)
    
    logging.info("Starting test with config file: " + args.configfile + "\n")
    
    # loading config script
    if config_path:
        sys.path.append(config_path)

    # updating config
    global config
    module_name = os.path.basename(args.configfile).replace('.py', '')
    config = importlib.import_module(module_name)

    return args

def run_tests(apicases):
    # run tests
    for _, apicase in enumerate(apicases):
        name = apicase.get_name()
        logging.info("[" + name + "]: " + apicase.get_url())

        # pre request
        apicase = config.pre_request(apicase)

        # send HTTP request
        apicase = config.do_request(apicase)
        
        # post request
        config.post_request(apicase)
        if apicase.get_response() is None:
            continue

        json_obj = config.parse_response(apicase)
        logging.debug("\tJSON object:" + str(json_obj))

        # run extractor
        logging.info("\tEXTRACTOR: ")
        apicase.extractor(json_obj)
        # run assertor
        logging.info("\tASSERTIONS: ")
        apicase.assertor(json_obj)
    
    return apicases

def parse_apicases_from_yaml(config, file):
    """
    parse apicases from yaml
    """
    apicases = []
    with open(file, 'r', encoding='utf-8') as f:
        data = yaml.safe_load(f.read())
        if not data.get('apicases'):
            return apicases
        
        for apiinfo in data.get('apicases'):
            parsed = parse_dict_with_extracts(config, apiinfo)

            # setup apicase
            apicase = APICase(config, parsed.get('name'), parsed.get('url'), parsed.get('method'))
            apicase.set_headers(parsed.get('headers'))
            apicase.set_data(parsed.get('data'))
            apicase.set_extracts(parsed.get('extracts'))
            apicase.set_assertions(parsed.get('assertions'))

            apicases.append(apicase)

    return apicases

def aft():
    args = initial()

    # pre testing
    config.pre_testing()

    # run tests
    pattern = (args.testcases + "/**/*.yaml") if os.path.isdir(args.testcases) else args.testcases
    files = sorted(glob.glob(pattern, recursive=True))

    results = []
    for i, filepath in enumerate(files):
        logging.info("{}:{}".format(i, filepath))
        apicases = parse_apicases_from_yaml(config, filepath)
        apicases = run_tests(apicases)
        results.append(apicases)
        logging.info("")
    
    with open(args.output, 'w', encoding='utf-8') as f:
        # save test result to file
        logging.info("Saved test result to file:" + args.output)
        json.dump([json.loads(str(api)) for api in results], f, ensure_ascii=False, indent=4)

    # post testing
    config.post_testing()
    
if __name__ != 'main':
    aft()



#!/bin/env python3

import uuid
import logging

# common headers
HEADERS = {
    'content-type': 'application/json',
    'x-token': '1234',
}

# common extracts
EXTRACTS = {
    'USERNAME':'allen'
}

def pre_testing():
    """
    pre testing
    """
    logging.debug("pre_testing()...")
    HEADERS['x-apiflows-id'] = str(uuid.uuid4())

def pre_request(apicase):
    """
    pre request
    """
    # TODO here
    logging.debug("pre_request()...")
    apicase.set_header("mycookie", "PHPSESSIONID=apiflows")
    return apicase

def do_request(apicase):
    logging.debug("do_request()...")
    apicase.do_request()
    return apicase

def parse_response(apicase):
    """
    parse HTTP response
    """
    logging.debug("parse_response()...")
    code = apicase.parse_status_code()
    headers = apicase.parse_response_headers()
    body = apicase.parse_response_body()
    return {'code': code, 'headers': headers, 'body': body}

def post_request(apicase):
    """
    post request
    """
    # TODO
    logging.debug("post_request()...")
    response = apicase.get_response()
    if response is not None:
        logging.debug("    RESPONSE: " + response.text)
    
    return apicase

def post_testing():
    """
    post testing
    """
    # TODO
    logging.debug("post_testing()...")